"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
class DiffFinder {
    static findDifferences(specs) {
        const topLevelXPropertiesDiffs = this.findDiffsInXProperties(specs.sourceSpec.xProperties, specs.destinationSpec.xProperties, 'xProperties');
        return Promise.resolve(topLevelXPropertiesDiffs);
    }
    static findDiffsInXProperties(sourceParsedXProperties, destinationParsedXProperties, xPropertyContainerName) {
        const xPropertyUniqueNames = _(_.keys(sourceParsedXProperties))
            .concat(_.keys(destinationParsedXProperties))
            .uniq()
            .value();
        const xPropertyDiffs = _(xPropertyUniqueNames)
            .map((xPropertyName) => {
            return this.findDiffsInProperty(sourceParsedXProperties[xPropertyName], destinationParsedXProperties[xPropertyName], `${xPropertyContainerName}.${xPropertyName}`);
        })
            .flatten()
            .value();
        return xPropertyDiffs;
    }
    static findDiffsInProperty(sourceObject, destinationObject, propertyName) {
        const additionDiffs = this.findAdditionDiffsInProperty(sourceObject, destinationObject, propertyName);
        const deletionDiffs = this.findDeletionDiffsInProperty(sourceObject, destinationObject, propertyName);
        const editionDiffs = this.findEditionDiffsInProperty(sourceObject, destinationObject, propertyName);
        return _.concat([], additionDiffs, deletionDiffs, editionDiffs);
    }
    static findAdditionDiffsInProperty(sourceObject, destinationObject, propertyName) {
        const isAddition = this.isUndefinedDeep(sourceObject) && this.isDefinedDeep(destinationObject);
        if (isAddition) {
            return [this.createDifference({ sourceObject, destinationObject, propertyName, action: 'add' })];
        }
        return [];
    }
    static findDeletionDiffsInProperty(sourceObject, destinationObject, propertyName) {
        const isDeletion = this.isDefinedDeep(sourceObject) && this.isUndefinedDeep(destinationObject);
        if (isDeletion) {
            return [this.createDifference({ sourceObject, destinationObject, propertyName, action: 'remove' })];
        }
        return [];
    }
    static findEditionDiffsInProperty(sourceObject, destinationObject, propertyName) {
        const isEdition = this.isDefinedDeep(sourceObject)
            && this.isDefinedDeep(destinationObject) && !_.isEqual(sourceObject.value, destinationObject.value);
        if (isEdition) {
            return [
                this.createDifference({ sourceObject, destinationObject, propertyName, action: 'add' }),
                this.createDifference({ sourceObject, destinationObject, propertyName, action: 'remove' })
            ];
        }
        return [];
    }
    static createDifference(options) {
        const entity = this.findEntityForDiff(options.propertyName);
        return {
            action: options.action,
            code: `${entity}.${options.action}`,
            destinationSpecEntityDetails: this.createSpecEntityDetails(options.destinationObject),
            entity,
            source: 'openapi-diff',
            sourceSpecEntityDetails: this.createSpecEntityDetails(options.sourceObject)
        };
    }
    static findEntityForDiff(propertyName) {
        return propertyName.includes('xProperties')
            ? 'unclassified'
            : `${propertyName}`;
    }
    static createSpecEntityDetails(parsedProperty) {
        return parsedProperty
            ? {
                location: parsedProperty.originalPath.join('.'),
                value: parsedProperty.value
            }
            : {
                location: undefined,
                value: undefined
            };
    }
    static isUndefinedDeep(objectWithValue) {
        return _.isUndefined(objectWithValue) || _.isUndefined(objectWithValue.value);
    }
    static isDefinedDeep(objectWithValue) {
        return this.isDefined(objectWithValue) && this.isDefined(objectWithValue.value);
    }
    static isDefined(target) {
        return !_.isUndefined(target);
    }
}
exports.DiffFinder = DiffFinder;
