"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const result_type_finder_1 = require("./spec-differ/result-type-finder");
const findEntityForDiff = (propertyName) => {
    return propertyName.includes('xProperties') ? 'unclassified' : propertyName;
};
const createDiffResult = (options) => {
    const entity = findEntityForDiff(options.propertyName);
    const code = `${entity}.${options.action}`;
    return {
        action: options.action,
        code,
        destinationSpecEntityDetails: createSpecEntityDetails(options.destinationObject),
        entity,
        source: 'openapi-diff',
        sourceSpecEntityDetails: createSpecEntityDetails(options.sourceObject),
        type: result_type_finder_1.resultTypeFinder.lookup(code)
    };
};
const createSpecEntityDetails = (parsedProperty) => {
    return parsedProperty
        ? {
            location: parsedProperty.originalPath.join('.'),
            value: parsedProperty.value
        }
        : {
            location: undefined,
            value: undefined
        };
};
const isDefined = (target) => {
    return !_.isUndefined(target);
};
const isDefinedDeep = (objectWithValue) => {
    return isDefined(objectWithValue) && isDefined(objectWithValue.value);
};
const isUndefinedDeep = (objectWithValue) => {
    return _.isUndefined(objectWithValue) || _.isUndefined(objectWithValue.value);
};
const findAdditionDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const isAddition = isUndefinedDeep(sourceObject) && isDefinedDeep(destinationObject);
    if (isAddition) {
        return [createDiffResult({ sourceObject, destinationObject, propertyName, action: 'add' })];
    }
    return [];
};
const findDeletionDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const isDeletion = isDefinedDeep(sourceObject) && isUndefinedDeep(destinationObject);
    if (isDeletion) {
        return [createDiffResult({ sourceObject, destinationObject, propertyName, action: 'remove' })];
    }
    return [];
};
const findEditionDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const isEdition = isDefinedDeep(sourceObject)
        && isDefinedDeep(destinationObject) && !_.isEqual(sourceObject.value, destinationObject.value);
    if (isEdition) {
        return [
            createDiffResult({ sourceObject, destinationObject, propertyName, action: 'add' }),
            createDiffResult({ sourceObject, destinationObject, propertyName, action: 'remove' })
        ];
    }
    return [];
};
const findDiffsInProperty = (sourceObject, destinationObject, propertyName) => {
    const additionDiffs = findAdditionDiffsInProperty(sourceObject, destinationObject, propertyName);
    const deletionDiffs = findDeletionDiffsInProperty(sourceObject, destinationObject, propertyName);
    const editionDiffs = findEditionDiffsInProperty(sourceObject, destinationObject, propertyName);
    return _.concat([], additionDiffs, deletionDiffs, editionDiffs);
};
const isValueInArray = (object, array) => {
    return _.some(array, { value: object.value });
};
const findAdditionDiffsInArray = (sourceArrayContent, destinationArrayContent, arrayName) => {
    const arrayContentAdditionDiffs = _(destinationArrayContent)
        .filter((entry) => {
        return !isValueInArray(entry, sourceArrayContent);
    })
        .map((addedEntry) => {
        return createDiffResult({
            action: 'add',
            destinationObject: addedEntry,
            propertyName: `${arrayName}.item`,
            sourceObject: undefined
        });
    })
        .flatten()
        .value();
    return arrayContentAdditionDiffs;
};
const findDeletionDiffsInArray = (sourceArrayContent, destinationArrayContent, arrayName) => {
    const arrayContentDeletionDiffs = _(sourceArrayContent)
        .filter((entry) => {
        return !isValueInArray(entry, destinationArrayContent);
    })
        .map((removedEntry) => {
        return createDiffResult({
            action: 'remove',
            destinationObject: undefined,
            propertyName: `${arrayName}.item`,
            sourceObject: removedEntry
        });
    })
        .flatten()
        .value();
    return arrayContentDeletionDiffs;
};
const findDiffsInArray = (sourceArray, destinationArray, objectName) => {
    const arrayAdditionDiffs = findAdditionDiffsInProperty(sourceArray, destinationArray, objectName);
    const arrayDeletionDiffs = findDeletionDiffsInProperty(sourceArray, destinationArray, objectName);
    let arrayContentAdditionDiffs = [];
    if (!arrayAdditionDiffs.length) {
        const sourceArrayContent = sourceArray.value;
        const destinationArrayContent = destinationArray.value;
        arrayContentAdditionDiffs = findAdditionDiffsInArray(sourceArrayContent, destinationArrayContent, objectName);
    }
    let arrayContentDeletionDiffs = [];
    if (!arrayDeletionDiffs.length) {
        const sourceArrayContent = sourceArray.value;
        const destinationArrayContent = destinationArray.value;
        arrayContentDeletionDiffs = findDeletionDiffsInArray(sourceArrayContent, destinationArrayContent, objectName);
    }
    return _.concat([], arrayAdditionDiffs, arrayDeletionDiffs, arrayContentAdditionDiffs, arrayContentDeletionDiffs);
};
const findDiffsInXProperties = (sourceParsedXProperties, destinationParsedXProperties, xPropertyContainerName) => {
    const xPropertyUniqueNames = _(_.keys(sourceParsedXProperties))
        .concat(_.keys(destinationParsedXProperties))
        .uniq()
        .value();
    const xPropertyDiffs = _(xPropertyUniqueNames)
        .map((xPropertyName) => {
        return findDiffsInProperty(sourceParsedXProperties[xPropertyName], destinationParsedXProperties[xPropertyName], `${xPropertyContainerName}.${xPropertyName}`);
    })
        .flatten()
        .value();
    return xPropertyDiffs;
};
const findDiffsInSpecs = (parsedSourceSpec, parsedDestinationSpec) => {
    const basePathDiffs = findDiffsInProperty(parsedSourceSpec.basePath, parsedDestinationSpec.basePath, 'basePath');
    const schemesDiffs = findDiffsInArray(parsedSourceSpec.schemes, parsedDestinationSpec.schemes, 'schemes');
    const topLevelXPropertyDiffs = findDiffsInXProperties(parsedSourceSpec.xProperties, parsedDestinationSpec.xProperties, 'xProperties');
    return Promise.resolve(_.concat([], basePathDiffs, schemesDiffs, topLevelXPropertyDiffs));
};
exports.specFinder = {
    diff: findDiffsInSpecs
};
